#!/bin/sh
malia=arm64
malim=mali450
malis=meson
maliv=r7p0

#brsub=buildroot_openlinux/buildroot/package/meson-mali
#brtgz=arm-buildroot-2016-08-18-5aaca1b35f.tar.gz
#brsub=buildroot_openlinux/buildroot/package/meson-mali
#brtgz=buildroot_openlinux_kernel_3.14_wayland_20170630.tar.gz
#brsub=buildroot_openlinux_kernel_4.9_fbdev_20180418/vendor/amlogic/meson_mali
#brtgz=buildroot_openlinux_kernel_4.9_fbdev_20180418.tar.gz
brsub=OpenLinux_20180907/vendor/amlogic/meson_mali
brtgz=buildroot_openlinux_kernel_4.9_20180907.tar.gz

if [ "X$1" = "Xarmhf" -o "X$1" = "Xarm64" ]; then
  malia=$1
fi

brarc=eabihf

if [ "${malia}" = "arm64" ]; then
  brarc=arm64
fi

if [ "X$3" = "Xmali400" -o "X$3" = "Xmali450" ]; then
  malim=$3
fi

brmal=m400
malil=meson_gx_mali_400

if [ "${malim}" = "mali450" ]; then
  brmal=m450
  malil=meson_gx_mali_450
fi

if [ "X$2" = "Xr5p1" -o "X$2" = "Xr6p1" -o "X$2" = "Xr7p0" ]; then
  maliv=$2
fi

brver=${maliv}
malix=${maliv}

if [ "${maliv:0:3}" = "r7p" ]; then
  brver=r7p0
elif [ "${maliv:0:3}" = "r6p" ]; then
  brver=r6p1
  malix=r6p2
elif [ "${maliv:0:3}" = "r5p" ]; then
  brver=r5p1
  malix=r7p0
fi

defcfg=arch/arm64/configs/odroidc2_defconfig

KDIR=$(pwd)/linux

scriptdir=`dirname $0`
cd ${scriptdir} ; scriptdir=`pwd` ; cd - > /dev/null

mkdir -p archives

cd archives

if [ ! -e ${brtgz} -a ! -e ${scriptdir}/meson-mali-lib ]; then
  #wget -N http://openlinux.amlogic.com:8000/download/ARM/filesystem/${brtgz}
  wget -N http://openlinux2.amlogic.com:8000/download/ARM/filesystem/Linux_BSP/${brtgz}
  #wget -N http://openlinux.amlogic.com:8000/download/ARM/filesystem/Linux_BSP/${brtgz}
fi

cd ..

if [ ! -e meson-mali-lib ]; then
  mkdir meson-mali-lib
  cd meson-mali-lib

  echo "Extracting meson mali lib from buildroot..."
  tar xzf ../archives/${brtgz} ${brsub}

  cd ..
fi

#if [ ! -e xf86-video-armsoc ]; then
#  git clone -b meson-drm https://github.com/superna9999/xf86-video-armsoc
#fi

if [ ! -e ${malil} ]; then
  if [ ${malix} = r6p2 ]; then
    git clone -b DX910-SW-99002-r6p1-01rel0_meson_gx https://github.com/superna9999/meson_gx_mali_450 ${malil}
  else
    git clone -b DX910-SW-99002-r7p0-00rel1_meson_gx https://github.com/superna9999/meson_gx_mali_450 ${malil}
  fi

  cd ${malil}/

  mkdir -p patches/${malix}
  for f in `cat ../patches/series-mali-${malix} | grep -E "^${malix}/" | cut -d / -f 2-` ; do
    cp -p ../patches/$f patches/${malix}/
    echo "${malix}/$f" >> patches/${malix}/series
  done

  ##patch -p1 < ../${malil}-4.14-build.patch
  #patch -p1 < ../${malil}-4.14.patch

  mkdir -p driver/patches
  #cp -p patches/*.patch driver/patches/
  cp -apP patches/${malix} driver/patches/
  mv driver/patches/${malix}/series  driver/patches/
  cd driver/
  quilt push -a
  cd ..

else
  cd ${malil}/
fi

cp -p ../patches/mali-lib-END_USER_LICENCE_AGREEMENT.txt ./END_USER_LICENCE_AGREEMENT.txt

mkdir -p ${malia}
cd ${malia}

echo "Prepare ${brarc}/${brmal}/${brver}"

# x11

if [ -e ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}-X/${brver}/libUMP.so ]; then
  mkdir -p lib_x11-ump
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}-X/${brver}/libMali.so lib_x11-ump/
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}-X/${brver}/libUMP.so  lib_x11-ump/

elif [ -e ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brver}/${brmal}-X/libUMP.so ]; then
  mkdir -p lib_x11-ump
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brver}/${brmal}-X/libMali.so lib_x11-ump/
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brver}/${brmal}-X/libUMP.so  lib_x11-ump/

elif [ -e ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}-X/${brver}/libMali.so ]; then
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}-X/${brver}/libMali.so .

elif [ -e ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brver}/${brmal}-X/libMali.so ]; then
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brver}/${brmal}-X/libMali.so .

else
  echo "Warning: ${brmal}-X/${brver}/libMali.so not found."
fi

# wayland

if [ -e ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}/${brver}/wayland/drm/libMali.so ]; then
  mkdir -p lib_wayland-drm
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}/${brver}/wayland/drm/libMali.so lib_wayland-drm/

else
   echo "Warning: ${brmal}/${brver}/wayland/drm/libMali.so not found."
fi

if [ -e  ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}/${brver}/wayland/fbdev/libMali.so ]; then
  mkdir -p lib_wayland-fbdev
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}/${brver}/wayland/fbdev/libMali.so lib_wayland-fbdev/

elif [ -e  ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brver}/${brmal}/wayland/libMali.so ]; then
  mkdir -p lib_wayland-fbdev
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brver}/${brmal}/wayland/libMali.so lib_wayland-fbdev/

else
   echo "Warning: ${brmal}/${brver}/wayland/fbdev/libMali.so not found."
fi

# fbdev

mkdir -p lib_fbdev
if [ -e ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}/${brver}/fbdev/libMali.so ]; then
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}/${brver}/fbdev/libMali.so lib_fbdev/

elif [ -e ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brver}/${brmal}/libMali.so ]; then
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brver}/${brmal}/libMali.so lib_fbdev/

else
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}/${brver}/libMali.so lib_fbdev/
fi

# dummy

if [ -e  ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}/${brver}/dummy/libMali.so ]; then
  mkdir -p lib_dummy
  cp -p ${scriptdir}/meson-mali-lib/${brsub}/lib/${brarc}/${brmal}/${brver}/dummy/libMali.so lib_dummy/

else
   echo "Warning: ${brmal}/${brver}/dummy/libMali.so not found."
fi

cd ..

mkdir -p ${scriptdir}/meson-mali-lib/${brsub}/patches
cp -p ../patches/mali-r7p0-dev-*.patch ${scriptdir}/meson-mali-lib/${brsub}/patches/
cp -p ../patches/series-mali-r7p0-dev ${scriptdir}/meson-mali-lib/${brsub}/patches/

cd ${scriptdir}/meson-mali-lib/${brsub}/
tar czf ${malim}-${malis}-dev.tar.gz include lib/pkgconfig patches
cd - > /dev/null

mv ${scriptdir}/meson-mali-lib/${brsub}/${malim}-${malis}-dev.tar.gz ../

cd ..

if [ -e ${KDIR}/drivers/gpu ]; then
  echo "Integrating meson mali driver..."

  if [ ! -e ${KDIR}/drivers/gpu/arm/mali ]; then
    mkdir -p ${KDIR}/drivers/gpu/arm
    cp -apP ${malil}/driver/src/devicedrv/mali ${KDIR}/drivers/gpu/arm/
    cp -apP ${malil}/driver/src/devicedrv/ump linux/drivers/gpu/arm/
    cp -apP ${malil}/driver/src/devicedrv/umplock linux/drivers/gpu/arm/
    #cp -apP ${malil}/driver/src/ump/include linux/drivers/gpu/arm/ump/
    cp -apP ${malil}/driver/src/ump/include/ump linux/include/
  fi

  if ! grep -q CONFIG_MALI400 ${KDIR}/${defcfg} ; then
    cat <<EOFMALIDEFCFG | tee -a ${KDIR}/${defcfg}

CONFIG_MALI400=m
CONFIG_MALI450=y
# CONFIG_MALI470 is not set
# CONFIG_MALI400_DEBUG is not set
# CONFIG_MALI400_PROFILING is not set
CONFIG_MALI400_UMP=y
# CONFIG_MALI_DVFS is not set
CONFIG_MALI_DMA_BUF_MAP_ON_ATTACH=y
# CONFIG_MALI_SHARED_INTERRUPTS is not set
# CONFIG_MALI_PMU_PARALLEL_POWER_UP is not set
# CONFIG_MALI_DT is not set
# CONFIG_MALI_DEVFREQ is not set
# CONFIG_MALI_QUIET is not set
EOFMALIDEFCFG
  fi

  if ! grep -q CONFIG_UMP ${KDIR}/${defcfg} ; then
    cat <<EOFUMPDEFCFG | tee -a ${KDIR}/${defcfg}

CONFIG_UMP=y
# CONFIG_UMP_DEBUG is not set
EOFUMPDEFCFG
  fi

  sed -i s/CONFIG_ARCH_EXYNOS4/CONFIG_ARCH_MESON/g ${KDIR}/drivers/gpu/arm/mali/Kbuild
  sed -i s/exynos4/meson/g ${KDIR}/drivers/gpu/arm/mali/Kbuild

  if [ ! -e ${KDIR}/drivers/gpu/arm/Kconfig ]; then
    touch ${KDIR}/drivers/gpu/arm/Kconfig
  fi
  if ! grep -q 'arm/mali/' ${KDIR}/drivers/gpu/arm/Kconfig ; then
    echo 'source "drivers/gpu/arm/mali/Kconfig"' >> ${KDIR}/drivers/gpu/arm/Kconfig
  fi
  if ! grep -q 'arm/ump/' ${KDIR}/drivers/gpu/arm/Kconfig ; then
    echo 'source "drivers/gpu/arm/ump/Kconfig"' >> ${KDIR}/drivers/gpu/arm/Kconfig
  fi

  if [ ! -e ${KDIR}/drivers/gpu/arm/Makefile ]; then
    touch ${KDIR}/drivers/gpu/arm/Makefile
  fi
  if ! grep -q 'mali/' ${KDIR}/drivers/gpu/arm/Makefile ; then
    echo 'obj-$(CONFIG_MALI400) += mali/' >> ${KDIR}/drivers/gpu/arm/Makefile
  fi
  if ! grep -q 'ump/' ${KDIR}/drivers/gpu/arm/Makefile ; then
    echo 'obj-$(CONFIG_UMP) += ump/' >> ${KDIR}/drivers/gpu/arm/Makefile
  fi
  if ! grep -q 'umplock/' ${KDIR}/drivers/gpu/arm/Makefile ; then
    echo 'obj-$(CONFIG_UMP) += umplock/' >> ${KDIR}/drivers/gpu/arm/Makefile
  fi

  if ! grep -q 'gpu/arm/' ${KDIR}/drivers/video/Kconfig ; then
    echo 'source "drivers/gpu/arm/Kconfig"' >> ${KDIR}/drivers/video/Kconfig
  fi

  if ! grep -q 'arm/' ${KDIR}/drivers/gpu/Makefile ; then
    echo 'obj-y += arm/' >> ${KDIR}/drivers/gpu/Makefile
  fi
fi

if [ -e mali450-${malis} ]; then
  if [ ! -e ${malim}-${malis} ]; then
    mkdir ${malim}-${malis}
    cp -apP mali450-${malis}/DEBIAN ${malim}-${malis}/
    sed -i s/mali450-${malis}/${malim}-${malis}/g ${malim}-${malis}/DEBIAN/control
  fi
fi

sed -i s/'^malia=.*'/'malia='${malia}/g build-mali-driver.sh
sed -i s/'^malil=.*'/'malil='${malil}/g build-mali-driver.sh
sed -i s/'^malim=.*'/'malim='${malim}/g build-mali-driver.sh
sed -i s/'^malis=.*'/'malis='${malis}/g build-mali-driver.sh
sed -i s/'^maliv=.*'/'maliv='${maliv}/g build-mali-driver.sh

echo OK
